#include <X3DTK/kernel.h>
#include <X3DTK/memreleaser.h>
#include <X3DTK/X3D/scenegraph.h>
#include <X3DTK/X3D/scenesaver.h>
#include <X3DTK/X3D/meshbuilder.h>
#include <X3DTK/MESH/primitivemeshbuilder.h>
#include <X3DTK/MESH/normalsupdater.h>
#include <X3DTK/MESH/x3dbuilder.h>

#include <iostream>

using namespace X3DTK;
using namespace std;

int main(int argc, char *argv[])
{
  if (argc <= 2)
  {
    cerr << "usage: X3DWorldCoordTranslator input output" << endl;
    exit(0);
  }

  // DefaultLoader to load the default X3D Nodes.
  X3D::Loader *loader = Singleton<X3D::Loader>::getInstance();
  MemReleaser *releaser = Singleton<MemReleaser>::getInstance();
  X3D::SceneSaver *sceneSaver = Singleton<X3D::SceneSaver>::getInstance();
  X3D::MeshBuilder *meshbuilder = Singleton<X3D::MeshBuilder>::getInstance();
  MESH::PrimitiveMeshBuilder *primitivemeshbuilder = Singleton<MESH::PrimitiveMeshBuilder>::getInstance();
  MESH::NormalsUpdater *normalsupdater = Singleton<MESH::NormalsUpdater>::getInstance();
  MESH::X3DBuilder *x3dbuilder = Singleton<MESH::X3DBuilder>::getInstance();
  
  // Loads the scene.
  X3D::Scene *s = loader->load(argv[1], true);
  MESH::Scene *ms = meshbuilder->build(s);
  releaser->release(s);
  
  cout << "computing" << endl;
  primitivemeshbuilder->build(ms);
  cout << "computed" << endl;
  normalsupdater->updateNormals(ms);
  s = x3dbuilder->build(ms);
  sceneSaver->saveAs(s, argv[2]);

  releaser->release(s);
  releaser->release(ms);
  
  return 1;
}
